@@ -65,21 +65,21 @@ setMethod(
             for(i in 1L:ncol(rs))
                 ddc <- ddc + rs[,i]/P[,i]  * (Pfull[,i] - 3*Pfull[,i]^2 + 2*Pfull[,i]^3 -
                     Pfull[,i+1L] + 3*Pfull[,i+1L]^2 - 2*Pfull[,i+1L]^3) -
-                    rs[,i]/P[,i]^2 * D2 * (PQfull[,i] - PQfull[,i+1L])^2
+                    rs[,i]/P[,i]^2 * (PQfull[,i] - PQfull[,i+1L])^2
             hess[cind, cind] <- sum(ddc)
             for(i in 1L:nzetas)
                 hess[cind, nfact + i] <- hess[nfact + i, cind] <-
-                    sum((rs[,i]/P[,i] * D2 * (-Pfull[,i+1L] + 3*Pfull[,i+1L]^2 - 2*Pfull[,i+1L]^3) -
-                    rs[,i]/P[,i]^2 * D2 * (PQfull[,i] - PQfull[,i+1L]) * (-PQfull[,i+1L]) +
-                    rs[,i+1L]/P[,i+1L] * D2 * (Pfull[,i+1L] - 3*Pfull[,i+1L]^2 + 2*Pfull[,i+1L]^3) -
-                    rs[,i+1L]/P[,i+1L]^2 * D2 * (PQfull[,i+1L] - PQfull[,i+2L]) * (PQfull[,i+1L])))
+                    sum((rs[,i]/P[,i] * (-Pfull[,i+1L] + 3*Pfull[,i+1L]^2 - 2*Pfull[,i+1L]^3) -
+                    rs[,i]/P[,i]^2 * (PQfull[,i] - PQfull[,i+1L]) * (-PQfull[,i+1L]) +
+                    rs[,i+1L]/P[,i+1L] * (Pfull[,i+1L] - 3*Pfull[,i+1L]^2 + 2*Pfull[,i+1L]^3) -
+                    rs[,i+1L]/P[,i+1L]^2 * (PQfull[,i+1L] - PQfull[,i+2L]) * (PQfull[,i+1L])))
             for(j in 1L:nfact){
                 tmp <- 0
                 for(i in 1L:ncol(rs))
-                        tmp <- tmp + (rs[,i]/P[,i] * D2 * Theta[,j] *
+                        tmp <- tmp + (rs[,i]/P[,i] * Theta[,j] *
                                           (Pfull[,i] - 3*Pfull[,i]^2 + 2*Pfull[,i]^3 -
                                                Pfull[,i+1L] + 3*Pfull[,i+1L]^2 - 2*Pfull[,i+1L]^3) -
-                                 rs[,i]/P[,i]^2 * D2 * (PQfull[,i] - PQfull[,i+1L]) * Theta[,j] *
+                                 rs[,i]/P[,i]^2 * (PQfull[,i] - PQfull[,i+1L]) * Theta[,j] *
                                       (PQfull[,i] - PQfull[,i+1L]))
                 hess[cind, j] <- hess[j, cind] <- sum(tmp)
             }
@@ -428,7 +428,7 @@ setMethod(
     f = "Deriv",
     signature = signature(x = 'GroupPars', Theta = 'matrix'),
     definition = function(x, Theta, CUSTOM.IND, EM = FALSE, pars = NULL, itemloc = NULL,
-                          tabdata = NULL, prior = NULL, estHess=FALSE){
+                          tabdata = NULL, estHess=FALSE, prior = NULL){
         if(EM){
             grad <- rep(0, length(x@par))
             hess <- matrix(0, length(x@par), length(x@par))
@@ -452,8 +452,10 @@ setMethod(
             siglong <- x@par[-(1L:nfact)]
             sig <- matrix(0,nfact,nfact)
             selcov <- lower.tri(sig, diag=TRUE)
-            scores2 <- matrix(0, nrow(tabdata), sum(selcov))
-            thetas2 <- numeric(sum(selcov))
+            sig[selcov] <- siglong
+            if(nfact != 1L)
+                sig <- sig + t(sig) - diag(diag(sig))
+            prior <- mirt_dmvnorm(Theta, mu, sig)
             ret <- .Call('EAPgroup', log(itemtrace), tabdata, Theta, prior, mu, 
                          mirtClusterEnv$ncores)
             tmp <- cbind(ret$scores, ret$scores2) * r
@@ -584,7 +586,7 @@ EML2 <- function(x, Theta, pars, tabdata, itemloc, CUSTOM.IND){
     gpars <- ExtractGroupPars(obj)
     mu <- gpars$gmeans
     sigma <- gpars$gcov
-    prior <- mvtnorm::dmvnorm(Theta, mean=mu, sigma=sigma)
+    prior <- mirt_dmvnorm(Theta, mean=mu, sigma=sigma)
     prior <- prior/sum(prior)
     rlist <- Estep.mirt(pars=pars, tabdata=tabdata, Theta=Theta, prior=prior, itemloc=itemloc,
                         CUSTOM.IND=CUSTOM.IND)
