@@ -76,11 +76,11 @@
 #'   \code{\link{residuals-method}}, \code{\link{plot-method}},
 #'   \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{mirt.model}}, \code{\link{mirt}},
 #'   \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{mixedmirt}},
-#'   \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, 
-#'   \code{\link{M2}},
+#'   \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, \code{\link{M2}},
 #'   \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}, \code{\link{probtrace}},
 #'   \code{\link{boot.mirt}}, \code{\link{imputeMissing}}, \code{\link{itemfit}}, \code{\link{mod2values}},
 #'   \code{\link{simdata}}, \code{\link{createItem}}, \code{\link{mirtCluster}}, \code{\link{DIF}}
+#   \code{\link{DTF}}
 #' @keywords models
 #' @export multipleGroup
 #' @examples
@@ -143,7 +143,7 @@
 #' #############
 #' #DIF test for each item (using all other items as anchors)
 #' itemnames <- colnames(dat)
-#' refmodel <- multipleGroup(dat, models, group = group,
+#' refmodel <- multipleGroup(dat, models, group = group, SE=TRUE,
 #'                              invariance=c('free_means', 'free_varcov', itemnames))
 #'
 #' #loop over items (in practice, run in parallel to increase speed). May be better to use ?DIF
@@ -166,9 +166,12 @@
 #'                              invariance=c('free_means', 'free_varcov', 'intercepts',
 #'                              itemnames[-i]))
 #'
-#'
 #' (anovas <- lapply(estmodels, anova, object2=refmodel, verbose=FALSE))
 #'
+#' #quickly test with Wald test using DIF()
+#' mod_configural2 <- multipleGroup(dat, models, group = group, SE=TRUE)
+#' DIF(mod_configural2, which.par = c('a1', 'd'), Wald=TRUE, p.adjust = 'fdr')
+#'
 #' #############
 #' #multiple factors
 #'
@@ -234,17 +237,16 @@
 #' head(fs[["D1"]])
 #' fscores(mod_configural, method = 'EAPsum')
 #'
-#' # constrain slopes within each group to be equal (but not accross groups)
+#' # constrain slopes within each group to be equal (but not across groups)
 #' model2 <- mirt.model('F1 = 1-15
 #'                       CONSTRAIN = (1-15, a1)')
 #' mod_configural2 <- multipleGroup(dat, model2, group = group)
-#' plot(mod_configural2, type = 'infocontour')
 #' plot(mod_configural2, type = 'SE')
 #' plot(mod_configural2, type = 'RE')
 #' itemplot(mod_configural2, 10)
 #'
 #' ############
-#' ## empical histogram example (normal and bimodal groups)
+#' ## empirical histogram example (normal and bimodal groups)
 #' set.seed(1234)
 #' a <- matrix(rlnorm(50, .2, .2))
 #' d <- matrix(rnorm(50))
@@ -264,7 +266,7 @@
 #'
 #' }
 multipleGroup <- function(data, model, group, itemtype = NULL, guess = 0, upper = 1,
-                          SE = FALSE, SE.type = 'SEM', invariance = '', pars = NULL,
+                          SE = FALSE, SE.type = 'crossprod', invariance = '', pars = NULL,
                           method = 'EM', constrain = NULL, parprior = NULL, calcNull = TRUE,
                           draws = 5000, quadpts = NULL, TOL = NULL, grsm.block = NULL, rsm.block = NULL,
                           key = NULL, technical = list(), accelerate = TRUE, empiricalhist = FALSE,
