@@ -34,16 +34,18 @@ randef <- function(x, ndraws = 1000, thin = 10){
         OffTerm <- OffTerm(random, J=J, N=N)
     } else OffTerm <- matrix(0, 1, ncol(x@data))
     gstructgrouppars <- ExtractGroupPars(x@pars[[J+1L]])
+    CUSTOM.IND <- x@CUSTOM.IND
     for(i in 1L:20L){
         tmpTheta <- draw.thetas(theta0=tmpTheta, pars=x@pars, fulldata=x@fulldata,
                                 itemloc=x@itemloc, cand.t.var=x@cand.t.var,
                                 prior.t.var=gstructgrouppars$gcov, OffTerm=OffTerm,
-                                prior.mu=gstructgrouppars$gmeans, prodlist=list())
+                                prior.mu=gstructgrouppars$gmeans, prodlist=list(),
+                                CUSTOM.IND=CUSTOM.IND)
         if(length(random) > 0L){
             for(j in 1L:length(random))
                 random[[j]]@drawvals <- DrawValues(random[[j]], Theta=tmpTheta, itemloc=x@itemloc,
                                                    pars=x@pars, fulldata=x@fulldata,
-                                                   offterm0=OffTerm)
+                                                   offterm0=OffTerm, CUSTOM.IND=CUSTOM.IND)
             OffTerm <- OffTerm(random, J=J, N=N)
         }
     }
@@ -51,13 +53,14 @@ randef <- function(x, ndraws = 1000, thin = 10){
         tmpTheta <- draw.thetas(theta0=tmpTheta, pars=x@pars, fulldata=x@fulldata,
                                 itemloc=x@itemloc, cand.t.var=x@cand.t.var,
                                 prior.t.var=gstructgrouppars$gcov, OffTerm=OffTerm,
-                                prior.mu=gstructgrouppars$gmeans, prodlist=list())
+                                prior.mu=gstructgrouppars$gmeans, prodlist=list(),
+                                CUSTOM.IND=CUSTOM.IND)
         if(i %% thin == 0) Theta <- Theta + tmpTheta
         if(length(random) > 0L){
             for(j in 1L:length(random)){
                 random[[j]]@drawvals <- DrawValues(random[[j]], Theta=tmpTheta, itemloc=x@itemloc,
                                                    pars=x@pars, fulldata=x@fulldata,
-                                                   offterm0=OffTerm)
+                                                   offterm0=OffTerm, CUSTOM.IND=CUSTOM.IND)
                 if(i %% thin == 0) Random[[j]] <- Random[[j]] + random[[j]]@drawvals
             }
             OffTerm <- OffTerm(random, J=J, N=N)
