@@ -5,9 +5,9 @@ void _Estep(vector<double> &, vector<double> &, const vector<double> &,
     const vector<int> &, const IntegerMatrix &, const NumericMatrix &,
     const int &);
 
-void _Estepbfactor(vector<double> &, vector<double> &, const NumericMatrix &,
-    const vector<double> &, const vector<double> &, const vector<int> &,
-    const int &, const IntegerMatrix &, const IntegerMatrix &,
+void _Estepbfactor(vector<double> &, vector<double> &, vector<double> &, 
+    const NumericMatrix &, const vector<double> &, const vector<double> &, 
+    const vector<int> &, const int &, const IntegerMatrix &, const IntegerMatrix &,
     const vector<double> &);
 
 #endif
